from typing import Dict, Any, List, Tuple
from datetime import datetime, timezone
import logging
import os
from dotenv import load_dotenv
load_dotenv()
LOG_LEVEL = os.environ.get('LOG_LEVEL', 'INFO').upper()

import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=LOG_LEVEL
)

from helpers.ScoreSaberAPI import ScoreSaberAPI
from helpers.SimpleScoreSaberAPI import SimpleScoreSaberAPI

from saberlist.utils import load_history, save_history, normalize_difficulty_name, format_time_ago, prompt_for_player_id

def playlist_strategy_scoresaber_ranked(
    scoresaber_api: SimpleScoreSaberAPI,
    min_star: float = 7.0,
    max_star: float = 10.0,
    song_count: int = 40
) -> Tuple[List[Dict[str, Any]], str]:
    """
    Build and format a list of the latest ranked songs from ScoreSaber within the given star range,
    avoiding reusing the same song+difficulty.
    Returns:
        (playlist_data, playlist_identifier):
            playlist_data: A list of playlist songs.
            playlist_identifier: A formatted string identifying this playlist version.
    """
    history = load_history()
    # Set up the history keys if they don't exist
    history.setdefault('scoresaber_ranked', {})
    history.setdefault('playlist_counts', {})

    # Prepare a unique key for this strategy and update its usage count
    count_key = 'scoresaber_ranked'
    current_count = history['playlist_counts'].get(count_key, 0)
    new_count = current_count + 1
    history['playlist_counts'][count_key] = new_count

    # Fetch ScoreSaber ranked maps within the desired star range
    logging.debug(f"Fetching ScoreSaber ranked maps with star range [{min_star}, {max_star}]...")
    ranked_maps = scoresaber_api.get_ranked_maps(
        min_star=min_star,
        max_star=max_star
    )

    # Filter maps that are actually ranked, and sort by rankedDate descending (latest first)
    filtered_maps = [
        m for m in ranked_maps
        if m.get('ranked') is True and m.get('rankedDate') is not None
    ]
    filtered_maps.sort(key=lambda x: x['rankedDate'], reverse=True)
    logging.info(f"Retrieved {len(filtered_maps)} ranked maps from ScoreSaber in the star range.")

    playlist_data = []

    for map_data in filtered_maps:
        if len(playlist_data) >= song_count:
            logging.debug(f"Reached the desired song count: {song_count}.")
            break

        song_hash = map_data.get('songHash')
        difficulty_info = map_data.get('difficulty', {})
        difficulty_raw = difficulty_info.get('difficultyRaw')
        game_mode = difficulty_info.get('gameMode', 'Standard')

        if not song_hash or not difficulty_raw:
            logging.debug(f"Skipping map due to missing hash or difficulty info: {map_data}")
            continue

        # Normalize the difficulty name (reusing the helper from oldscores)
        difficulty = normalize_difficulty_name(difficulty_raw)

        # Avoid reusing the same song + difficulty
        if song_hash in history['scoresaber_ranked'] and difficulty in history['scoresaber_ranked'][song_hash]:
            logging.debug(f"Skipping song {song_hash} - {difficulty} as it's in history.")
            continue

        # Prepare the data for the playlist
        song_dict = {
            'hash': song_hash,
            'songName': map_data.get('songName', 'Unknown'),
            'difficulties': [
                {
                    'name': difficulty,
                    'characteristic': 'Standard' if 'Standard' in game_mode else game_mode
                }
            ]
        }
        playlist_data.append(song_dict)
        logging.debug(f"Added {song_dict['songName']} (Difficulty Raw: {difficulty_raw}) to the playlist.")

        # Update history to mark this song + difficulty combination as used
        history['scoresaber_ranked'].setdefault(song_hash, []).append(difficulty)

    # Log final results
    if playlist_data:
        logging.info(f"Added {len(playlist_data)} new songs to the playlist.")
    else:
        logging.info("No new ranked songs were added to the playlist based on history.")

    save_history(history)

    return playlist_data, f"scoresaber_ranked-{new_count:02d}"


def playlist_strategy_scoresaber_oldscores(
    api: ScoreSaberAPI, 
    song_count: int = 20
) -> List[Dict[str, Any]]:
    """Build and format a list of songs based on old scores from ScoreSaber, avoiding reusing the same song+difficulty."""
    
    player_id = prompt_for_player_id()
    history = load_history()
    history.setdefault('scoresaber_oldscores', {})
    history.setdefault('playlist_counts', {})

    # Get the current count for ScoreSaber old scores and increment it
    count_key = 'scoresaber_oldscores'
    current_count = history['playlist_counts'].get(count_key, 0)
    new_count = current_count + 1
    history['playlist_counts'][count_key] = new_count

    scores_data = api.get_player_scores(player_id, use_cache=True)
    all_scores = scores_data.get('playerScores', [])
    if not all_scores:
        logging.warning(f"No scores found for player ID {player_id}.")
        return [], ""
    logging.debug(f"Found {len(all_scores)} scores for player ID {player_id}.")
    
    # Sort scores by timeSet in ascending order (oldest first)
    all_scores.sort(key=lambda x: x['score'].get('timeSet', ''))
    
    playlist_data = []
    current_time = datetime.now(timezone.utc)
    
    for score in all_scores:
        leaderboard = score.get('leaderboard', {})
        song_id = leaderboard.get('songHash')
        difficulty_raw = leaderboard.get('difficulty', {}).get('difficultyRaw', '')
        
        if not song_id or not difficulty_raw:
            logging.debug(f"Skipping score due to missing song_id or difficulty_raw: {score}")
            continue  # Skip if essential data is missing
        
        # Calculate time ago
        time_set_str = score['score'].get('timeSet')
        if not time_set_str:
            logging.debug(f"Skipping score due to missing timeSet: {score}")
            continue  # Skip if time_set is missing
        try:
            time_set = datetime.fromisoformat(time_set_str.replace('Z', '+00:00'))
        except ValueError as e:
            logging.error(f"Invalid time format for score ID {score['score'].get('id')}: {e}")
            continue
        time_difference = current_time - time_set
        time_ago = format_time_ago(time_difference)
        
        # Normalize the difficulty name
        difficulty = normalize_difficulty_name(difficulty_raw)
        game_mode = leaderboard.get('difficulty', {}).get('gameMode', 'Standard')
        if 'Standard' in game_mode:
            game_mode = 'Standard'
        
        # Check history to avoid reusing song+difficulty
        if song_id in history['scoresaber_oldscores'] and difficulty in history['scoresaber_oldscores'][song_id]:
            logging.debug(f"Skipping song {song_id} with difficulty {difficulty} as it's in history.")
            continue  # Skip if already used
        
        # Format the song data as expected by PlaylistBuilder
        song_dict = {
            'hash': song_id,
            'songName': leaderboard.get('songName', 'Unknown'),
            'difficulties': [
                {
                    'name': difficulty,
                    'characteristic': game_mode
                }
            ]
        }
        
        # Add the song to the playlist
        playlist_data.append(song_dict)
        logging.debug(f"Selected song for playlist: {song_dict['songName']} ({difficulty})")
        
        # Log the song addition
        mapper = "Unknown"  # Mapper information can be added if available
        logging.info(f"Song added: {song_dict['songName']} ({difficulty}), mapped by {mapper}. Last played {time_ago} ago.")
        
        # Check if the desired number of songs has been reached
        if len(playlist_data) >= song_count:
            logging.debug(f"Reached the desired song count: {song_count}.")
            break
    
    # Log if no songs were added
    if not playlist_data:
        logging.info("No new songs found to add to the playlist based on history.")
    else:
        logging.info(f"Total songs added to playlist: {len(playlist_data)}")
    
    # Update history to avoid reusing the same song+difficulty
    for song in playlist_data:
        song_id = song['hash']
        difficulty_name = song['difficulties'][0]['name']
        history['scoresaber_oldscores'].setdefault(song_id, []).append(difficulty_name)
    save_history(history)
    
    return playlist_data, f"scoresaber_oldscores-{new_count:02d}"